import json


class SerializeJSON(object):

    """
    Core data handler for json output. Stores all final objects
    and allows a standard way to ingest data from SimplyDomain.

    Sample JSON Data Struc:

    JSON_OUTPUT = {
        "meta_data": {
            "version": 0.1,
            "author": "Alexander Rymdeko-Harvey",
            "twitter": "Killswitch-GUI",
            "github_repo": "https://github.com/killswitch-GUI/SimplyDomain"
        },


        "args": {
            "domain": "google.com",
            "verbose": false,
            "debug": false
        },

        "data": [
            {
                "name": "cert search",
                "module_name": "crtsh_search.py",
                "source": "github.com",
                "time": 1222121212.11,
                "subdomain": "test.test.com"
            },
            {
                "name": "cert search",
                "module_name": "crtsh_search.py",
                "source": "github.com",
                "time": 1222121212.11,
                "subdomain": "test.test.com"
            }
        ]
    }
    """

    def __init__(self, config):
        """
        Init class struc. Used as a object to parse and populate
        results.
        """
        self.subdomains = {}
        self.subdomains['args'] = {}
        self.subdomains['data'] = []
        self.subdomains['meta_data'] = config['meta_data']
        self.subdomains['args']['domain'] = config['args'].DOMAIN
        self.subdomains['args']['debug'] = config['args'].debug
        self.subdomains['args']['verbose'] = config['args'].verbose

    def add_subdomain(self, input_obj):
        """
        Add subdomain to class kbject for storage untill output is needed.
        :param input_obj: class object of subdomain
        :param config: core .config.json and params loaded
        :return: 
        """
        subdomain = {}
        subdomain['name'] = input_obj.name
        subdomain['module_name'] = input_obj.module_name
        subdomain['module_version'] = input_obj.module_version
        subdomain['source'] = input_obj.source
        subdomain['time'] = input_obj.time
        subdomain['toolname'] = input_obj.toolname
        subdomain['subdomain'] = input_obj.subdomain
        subdomain['valid'] = input_obj.valid
        self.subdomains['data'].append(subdomain)

    def print_json_subdomains(self):
        """
        Simple test print.
        :return: 
        """
        json_str = json.dumps(self.subdomains, sort_keys=True, indent=4)
        print(json_str)


class SubDomain(object):

    """
    Core data handler to clean, and post results in proper
    DataSerialization format for SimplyDomain.

    Attributes:
        name: long name of method
        module_name: name of the module that performed collection 
        source: source of the subdomain or resource of collection
        module_version: version from meta
        source: source of the collection
        time: time the result obj was built
        toolname: tool used to collect data
        subdomain: subdomain to use
        valid: is domain valid
    """

    def __init__(self, name, module_name, module_version, source, time, subdomain, valid):
        """
        Init class struc. Used as a object to parse and populate
        results.
        """
        self.name = name
        self.module_name = module_name
        self.module_version = module_version
        self.source = source
        self.time = time
        self.toolname = "SimplyDomain"
        self.subdomain = subdomain
        self.valid = valid



import requests
import validators
from fake_useragent import UserAgent

from . import core_output


class RequestsHelpers(core_output.CoreOutput):

    """
    A set of functions to make requests standard and handle fail cases 
    to prevent error msgs.
    """

    def __init__(self):
        """
        Init class structure.
        """
        # TODO: Init Logging class
        core_output.CoreOutput.__init__(self)
        self.ua = UserAgent()

    def get_dns_wildcard(self, domain):
        try:
            x = "*.{}".format(domain)
            url = 'https://dns.google.com/resolve?name=%s&type=A' % (str(x))
            headers = {"Accept": "application/json"}
            response = requests.get(url,headers=headers,verify=True)
            return response.json()
        except Exception as e:
            print(response.text)
            return {}

    # split up json from raw for future support
    def request_json(self, url, return_code=200):
        """
        Request JSON content and expected return code.
        :param url:  URL to request
        :param return_code: expected return code or raise error
        :return: JSON, SuccessState
        """
        try:
            header = {
                'User-Agent': str(self.ua.google)
            }
            if not validators.url(url):
                self.print_yellow(
                    " [!] Invalid URL Requested: %s" % (str(url)))
                return {}, False
            r = requests.get(url, headers=header)
            if r.status_code != return_code:
                self.print_yellow(" [!] Request returned invalid status code: (CODE): %s (EXPECTED): %s" %
                                  (str(r.status_code), str(return_code)))
                return {}, False
            return r.content, True
        except requests.ConnectTimeout as e:
            self.print_red(
                " [!] Request ConnectionTimeout: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.TooManyRedirects as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.HTTPError as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except ConnectionError as e:
            self.print_red(
                " [!] Request ConnectionError: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except Exception as e:
            self.print_red(
                " [!] Request Unknown Error: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False

    def request_content(self, url, return_code=200):
        """
        Request content and expected return code.
        :param url:  URL to request
        :param return_code: expected return code or raise error
        :return: JSON, SuccessState
        """
        try:
            header = {
                'User-Agent': str(self.ua.google)
            }
            if not validators.url(url):
                self.print_yellow(
                    " [!] Invalid URL Requested: %s" % (str(url)))
                return {}, False
            r = requests.get(url, headers=header)
            if r.status_code != return_code:
                self.print_yellow(" [!] Request returned invalid status code: (CODE): %s (EXPECTED): %s" %
                                  (str(r.status_code), str(return_code)))
                return {}, False
            return r.content, True
        except requests.ConnectTimeout as e:
            self.print_red(
                " [!] Request ConnectionTimeout: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.TooManyRedirects as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.HTTPError as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except ConnectionError as e:
            self.print_red(
                " [!] Request ConnectionError: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except Exception as e:
            self.print_red(
                " [!] Request Unknown Error: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False

    def request_text(self, url, return_code=200):
        """
        Request content and expected return code.
        :param url:  URL to request
        :param return_code: expected return code or raise error
        :return: JSON, SuccessState
        """
        try:
            header = {
                'User-Agent': str(self.ua.google)
            }
            if not validators.url(url):
                self.print_yellow(
                    " [!] Invalid URL Requested: %s" % (str(url)))
                return {}, False
            r = requests.get(url, headers=header)
            if r.status_code != return_code:
                self.print_yellow(" [!] Request returned invalid status code: (CODE): %s (EXPECTED): %s" %
                                  (str(r.status_code), str(return_code)))
                return {}, False
            return r.text, True
        except requests.ConnectTimeout as e:
            self.print_red(
                " [!] Request ConnectionTimeout: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.TooManyRedirects as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.HTTPError as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except ConnectionError as e:
            self.print_red(
                " [!] Request ConnectionError: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except Exception as e:
            self.print_red(
                " [!] Request Unknown Error: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False

    def request_raw(self, url, return_code=200):
        """
        Request content and expected return code.
        :param url:  URL to request
        :param return_code: expected return code or raise error
        :return: JSON, SuccessState
        """
        try:
            header = {
                'User-Agent': str(self.ua.google)
            }
            if not validators.url(url):
                self.print_yellow(
                    " [!] Invalid URL Requested: %s" % (str(url)))
                return {}, False
            r = requests.get(url, headers=header)
            if r.status_code != return_code:
                self.print_yellow(" [!] Request returned invalid status code: (CODE): %s (EXPECTED): %s" %
                                  (str(r.status_code), str(return_code)))
                return {}, False
            return r, True
        except requests.ConnectTimeout as e:
            self.print_red(
                " [!] Request ConnectionTimeout: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.TooManyRedirects as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except requests.HTTPError as e:
            self.print_red(
                " [!] Request TooManyRedirects: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except ConnectionError as e:
            self.print_red(
                " [!] Request ConnectionError: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False
        except Exception as e:
            self.print_red(
                " [!] Request Unknown Error: (URL): %s (ERROR): %s" % (str(url), str(e)))
            return {}, False



import json


__json_config = {

    "meta_data": {
        "version": 0.1,
        "author": "Alexander Rymdeko-Harvey",
        "twitter": "Killswitch-GUI",
        "github_repo": "https://github.com/killswitch-GUI/SimplyDomain"
    },
    "subdomain_bruteforce": {
        "top_1000": [
            "third_party",
            "dnspop",
            "results",
            "bitquark_20160227_subdomains_popular_1000"
        ],
        "top_10000": [
            "third_party",
            "dnspop",
            "results",
            "bitquark_20160227_subdomains_popular_10000"
        ],
        "top_100000": [
            "third_party",
            "dnspop",
            "results",
            "bitquark_20160227_subdomains_popular_100000"
        ],
        "top_1000000": [
            "third_party",
            "dnspop",
            "results",
            "bitquark_20160227_subdomains_popular_1000000"
        ]
    },
    "yahoo_search": {
        "start_count": 0,
        "end_count": 1000,
        "quantity": 100,
        "sleep_time": 3,
        "sleep_jitter": 0.30
    },
    "bing_search": {
        "start_count": 1,
        "end_count": 100,
        "sleep_time": 3,
        "sleep_jitter": 0.30
    }

}



from termcolor import colored, cprint
import json


class CorePrinters(object):
    """
    Core class: handles all data output within the project.
    """

    __title_screen = """
    ------------------------------------------------------------
      ______  _______                                 __          
     /      \/       \                               /  |         
    /$$$$$$  $$$$$$$  | ______  _____  ____   ______ $$/ _______  
    $$ \__$$/$$ |  $$ |/      \/     \/    \ /      \/  /       \ 
    $$      \$$ |  $$ /$$$$$$  $$$$$$ $$$$  |$$$$$$  $$ $$$$$$$  |
     $$$$$$  $$ |  $$ $$ |  $$ $$ | $$ | $$ |/    $$ $$ $$ |  $$ |
    /  \__$$ $$ |__$$ $$ \__$$ $$ | $$ | $$ /$$$$$$$ $$ $$ |  $$ |
    $$    $$/$$    $$/$$    $$/$$ | $$ | $$ $$    $$ $$ $$ |  $$ |
     $$$$$$/ $$$$$$$/  $$$$$$/ $$/  $$/  $$/ $$$$$$$/$$/$$/   $$/ 
    ------------------------------------------------------------                                                                                              
    """
    __config_startup = """
 *----------------------------------*
 |   CONFIGURATION INITIALIZATION   |
 *----------------------------------*
        """

    __d_module_change = """
 *----------------------------------*
 |  DYNAMIC MODULES INITIALIZATION  |
 *----------------------------------*
    """
    __s_module_change = """
 *----------------------------------*
 |   STATIC MODULES INITIALIZATION  |
 *----------------------------------*
    """

    def __init__(self):
        """
        INIT class object and define
        statics.
        """
        self.print_green = lambda x: cprint(x, 'green')
        self.print_green_on_bold = lambda x: cprint(x, 'green', attrs=['bold'])
        self.print_yellow = lambda x: cprint(x, 'yellow')
        self.print_yellow_on_bold = lambda x: cprint(
            x, 'yellow', attrs=['bold'])
        self.print_red = lambda x: cprint(x, 'red')
        self.print_red_on_bold = lambda x: cprint(x, 'red', attrs=['bold'])
        self.print_white = lambda x: cprint(x, 'white')

    def blue_text(self, msg):
        """
        Return green text obj.
        :param msg: TEXT
        :return: OBJ
        """
        s = colored(' [*] ', color='blue')
        msg = s + msg
        return msg

    def green_text(self, msg):
        """
        Return green text obj.
        :param msg: TEXT
        :return: OBJ
        """
        s = colored(' [+] ', color='green')
        msg = s + msg
        return msg

    def print_entry(self):
        """
        Print entry screen to the project.
        :return: NONE
        """
        self.print_green_on_bold(self.__title_screen)

    def print_d_module_start(self):
        """
        Print entry to dynamic modules
        :return: 
        """
        self.print_yellow(self.__d_module_change)

    def print_s_module_start(self):
        """
        Print entry to dynamic modules
        :return:
        """
        self.print_yellow(self.__s_module_change)

    def print_config_start(self):
        """
        Print entry to dynamic modules
        :return:
        """
        self.print_yellow(self.__config_startup)

    def print_modules(self, module_list):
        """
        Print all modules within the framework to be run
        :param module_list: mod list of loaded functions 
        :return: NONE
        """
        self.print_red_on_bold(" [*] Available modules are:")
        x = 1
        ordlist = []
        finalList = []
        for name in module_list:
            parts = name.split("/")
            ordlist.append(parts[-1])
        ordlist = sorted(ordlist)
        for name in ordlist:
            name = 'modules/' + name
            finalList.append(name)
        for name in finalList:
            print("\t%s)\t%s" % (x, '{0: <24}'.format(name)))
            x += 1

    def print_modules_long(self, module_list):
        """
        Print all modules within the framework to be run
        :param module_list: mod list of loaded functions 
        :return: NONE
        """
        self.print_red_on_bold(" [*] Available modules are:")
        print("-" * 60)
        for mod in module_list:
            dynamic_module = module_list[mod]
            dm = dynamic_module.DynamicModule()
            parts = mod.split("/")
            name = 'modules/' + parts[-1]
            self.print_yellow_on_bold(
                " %s" % ('{0: <24}'.format(name).ljust(40)))
            print(json.dumps(dm.info, indent=4))
            print("-" * 60)



import os
import random
import queue
import asyncio
import aiodns
import functools
# disable uvloop until supports windows
import uvloop
import socket
import click
import time
import string
from tqdm import tqdm
from itertools import product

from simplydomain.src import core_serialization
from simplydomain.src import module_helpers
from simplydomain.src import core_scrub


# use RequestsHelpers() class to make requests to target URL
class DynamicModule(module_helpers.RequestsHelpers):
    """
    Dynamic module class that will be loaded and called
    at runtime. This will allow modules to easily be independent of the
    core runtime.
    """

    def __init__(self, json_entry):
        """
        Init class structure. Each module takes a JSON entry object which
        can pass different values to the module with out changing up the API.
        adapted form  Empire Project:
        https://github.com/EmpireProject/Empire/blob/master/lib/modules/python_template.py

        :param json_entry: JSON data object passed to the module.
        """
        module_helpers.RequestsHelpers.__init__(self)
        self.json_entry = json_entry
        self.info = {
            # mod name
            'Module': 'subdomain_bruteforce.py',

            # long name of the module to be used
            'Name': 'Recursive Subdomain Bruteforce Using Wordlist',

            # version of the module to be used
            'Version': '1.0',

            # description
            'Description': ['Uses lists from dnspop',
                            'with high quality dns resolvers.'],

            # authors or sources to be quoted
            'Authors': ['@Killswitch-GUI', '@blark'],

            # list of resources or comments
            'comments': [
                'Searches and performs recursive dns-lookup.',
                ' adapted from https://github.com/blark/aiodnsbrute/blob/master/aiodnsbrute/cli.py'
            ],
            # priority of module (0) being first to execute
            'priority': 0
        }

        self.options = {
        }
        # ~ queue object
        self.word_count = int(self.json_entry['args'].raw_depth)
        self.word_list_queue = queue.Queue(maxsize=0)
        self.tasks = []
        self.domain = ''
        self.errors = []
        self.fqdn = []
        self.sub_gen_count = 0
        # disable uvloop until supports windows
        # asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        self.loop = asyncio.get_event_loop()
        self.resolver = aiodns.DNSResolver(loop=self.loop, rotate=True)
        # TODO: make max tasks defined in config.json
        self.max_tasks = 1024
        # TODO: make total set from wordcount in config.json
        self.sem = asyncio.BoundedSemaphore(self.max_tasks)
        self.cs = core_scrub.Scrub()
        self.core_args = self.json_entry['args']
        self.core_resolvers = self.json_entry['resolvers']
        self.silent = self.json_entry['silent']

    def dynamic_main(self, queue_dict):
        """
        Main entry point for process to call.

        core_serialization.SubDomain Attributes:
            name: long name of method
            module_name: name of the module that performed collection
            source: source of the subdomain or resource of collection
            module_version: version from meta
            source: source of the collection
            time: time the result obj was built
            subdomain: subdomain to use
            valid: is domain valid

        :return: NONE
        """
        self.task_output_queue = queue_dict['task_output_queue']
        self.domain = str(self.core_args.DOMAIN)
        self._execute_resolve()

    async def _process_dns_wordlist(self):
        """
        Populates word list queue with words to brute
        force a domain with.
        :return: NONE
        """
        # fancy iter so we can pull out only (N) lines
        sub_doamins = []
        for x in range(1, self.word_count+1, 1):
            sub_doamins += [''.join(i) for i in product(
                string.ascii_lowercase + string.digits, repeat=x)]
        self.sub_gen_count = len(sub_doamins)
        # move pbar here to ensure we have proper count at start
        if not self.silent:
            self.pbar = tqdm(total=self.sub_gen_count,
                             unit="records", maxinterval=0.1, mininterval=0)
        for word in sub_doamins:
            # Wait on the semaphore before adding more tasks
            await self.sem.acquire()
            host = '{}.{}'.format(word.strip(), self.domain)
            task = asyncio.ensure_future(self._dns_lookup(host))
            task.add_done_callback(functools.partial(
                self._dns_result_callback, host))
            self.tasks.append(task)
        await asyncio.gather(*self.tasks, return_exceptions=True)

    def _select_random_resolver(self):
        """
        Select a random resolver from the JSON config, allows
        for procs to easily obtain a IP.
        :return: STR: ip
        """
        ip = random.choice(self.json_entry['resolvers'])
        return ip

    def _execute_resolve(self, recursive=True):
        """
        Execs a single thread based / adapted from:
        https://github.com/blark/aiodnsbrute/blob/master/aiodnsbrute/cli.py
        :return: NONE
        """
        try:
            self.logger("Brute forcing {} with a maximum of {} concurrent tasks...".format(
                self.domain, self.max_tasks))
            self.logger("Wordlist loaded, brute forcing {} DNS records".format(
                self.sub_gen_count))
            # TODO: enable verbose
            if recursive:
                self.logger("Using recursive DNS with the following servers: {}".format(
                    self.resolver.nameservers))
            else:
                domain_ns = self.loop.run_until_complete(
                    self._dns_lookup(self.domain, 'NS'))
                self.logger(
                    "Setting nameservers to {} domain NS servers: {}".format(self.domain, [host.host for host in domain_ns]))
                self.resolver.nameservers = [
                    socket.gethostbyname(host.host) for host in domain_ns]
                #self.resolver.nameservers = self.core_resolvers
            self.loop.run_until_complete(self._process_dns_wordlist())
        except KeyboardInterrupt:
            self.logger("Caught keyboard interrupt, cleaning up...")
            asyncio.gather(*asyncio.Task.all_tasks()).cancel()
            self.loop.stop()
        finally:
            self.loop.close()
            # TODO: enable verbose
            if not self.silent:
                self.pbar.close()
            self.logger(
                "completed, {} subdomains found.".format(len(self.fqdn)))
        return self.fqdn

    def logger(self, msg, msg_type='info', level=0):
        """A quick and dirty msfconsole style stdout logger."""
        # TODO: enable verbose
        if not self.silent:
            style = {'info': ('[*]', 'blue'), 'pos': ('[+]', 'green'), 'err': ('[-]', 'red'),
                     'warn': ('[!]', 'yellow'), 'dbg': ('[D]', 'cyan')}
            if msg_type is not 0:
                decorator = click.style('{}'.format(
                    style[msg_type][0]), fg=style[msg_type][1], bold=True)
            else:
                decorator = ''
            m = " {} {}".format(decorator, msg)
            tqdm.write(m)

    async def _dns_lookup(self, name, _type='A'):
        """Performs a DNS request using aiodns, returns an asyncio future."""
        response = await self.resolver.query(name, _type)
        return response

    def _dns_result_callback(self, name, future):
        """Handles the result passed by the _dns_lookup function."""
        # Record processed we can now release the lock
        self.sem.release()
        # Handle known exceptions, barf on other ones
        if future.exception() is not None:
            try:
                err_num = future.exception().args[0]
                err_text = future.exception().args[1]
            except IndexError:
                self.logger("Couldn't parse exception: {}".format(
                    future.exception()), 'err')
            if (err_num == 4):  # This is domain name not found, ignore it.
                pass
            elif (err_num == 12):  # Timeout from DNS server
                self.logger("Timeout for {}".format(name), 'warn', 2)
            elif (err_num == 1):  # Server answered with no data
                pass
            else:
                self.logger('{} generated an unexpected exception: {}'.format(
                    name, future.exception()), 'err')
            self.errors.append({'hostname': name, 'error': err_text})
        # Output result
        else:
            self.cs.subdomain = name
            # check if domain name is valid
            valid = self.cs.validate_domain()
            # build the SubDomain Object to pass
            sub_obj = core_serialization.SubDomain(
                self.info["Name"],
                self.info["Module"],
                "",
                self.info["Version"],
                time.time(),
                name,
                valid
            )
            self.task_output_queue.put(sub_obj)
            ip = ', '.join([ip.host for ip in future.result()])
            self.fqdn.append((name, ip))
            # self.logger("{:<30}\t{}".format(name, ip), 'pos')
            # self.logger(future.result(), 'dbg', 3)
        self.tasks.remove(future)
        # TODO: enable verbose
        if not self.silent:
            self.pbar.update()



import multiprocessing as mp
from itertools import product
import string
import threading
import time

from . import core_printer
from . import core_progress
from . import core_serialization
from . import module_recursion


class CoreProcess(core_printer.CorePrinters, core_progress.CoreProgress, module_recursion.ModuleRecursion):
    """
    Core class to handle threading and process 
    creation.
    """

    def __init__(self):
        """class init
        
        setups for a full MP stack.
        """
        core_printer.CorePrinters.__init__(self)
        core_progress.CoreProgress.__init__(self)
        module_recursion.ModuleRecursion.__init__(self)
        self.procs = []
        self.threads = []
        self.processors = mp.cpu_count()
        self.module_count = 0
        self.mp = mp
        self.mpq = self.mp.Queue()
        self.task_queue = mp.Queue()
        self.task_output_queue = mp.Queue()
        self.task_msg_queue = mp.Queue()
        self.progress_bar_pickup = mp.Queue()

        # output handlers
        self.serialize_json_output = core_serialization.SerializeJSON(
            self.config)

    def _configure_mp(self):
        """
        Sets the configuration props for 
        mp to handle taskings.
        :return: NONE
        """
        # use SPAWN since FORK is not supported on windows
        # SPAWN is slower since it creates a entire python
        # interpter

        self.logger.infomsg(
            'setting MP to SPAWN for cross platform support', 'CoreRuntime')
        self.mp.set_start_method('spawn')

    def _configure_processes(self, mod_count):
        """
        Check to make sure we dont start
        more procs than needed. Also fill taskQ with
        None values (PILLS)
        :return: NONE
        """
        self.logger.infomsg('current # of modules loaded: ' +
                            str(mod_count), 'CoreProcess')
        self.logger.infomsg('current # of processors of SYSTEM: ' +
                            str(self.processors), 'CoreProcess')
        if self.processors > mod_count:
            self.logger.infomsg('setting MP count of: ' +
                                str(mod_count), 'CoreProcess')
            self.processors = mod_count
        # populate the PILL == None
        for p in range(self.processors + 1):
            self.task_queue.put(None)

    def _task_output_queue_consumer(self):
        """
        Consumes task_output_queue data at set interval,
        to be run as a Thread at a interval of 1 sec. If this res
        is to low queue mem could lock.
        :return: NONE
        """
        while True:
            item = self.task_output_queue.get()
            if item:
                self.logger.debugmsg('_task_output_queue_consumer recv a subdomain: '
                                     + str(item.subdomain), 'CoreProcess')
                msg = self.green_text("Subdomain: %s Vaild: (%s)" %
                                      ('{0: <30}'.format('('+str(item.subdomain)+')'), str(item.valid)))
                if not self.config['silent']:
                    self.progress_print(msg)
                self.serialize_json_output.add_subdomain(item)
                self.add_subdomain(item)
            if item == None:
                self.logger.infomsg(
                    '_task_output_queue_consumer is (NONE) exiting thread', 'CoreProcess')
                break
            if self.task_output_queue.empty():
                pass
                # time.sleep(0.1)

    def populate_task_queue(self, modules):
        """
        Populats the queue of module objects to be executed.
        :param modules: passed list of dynamic loaded modules
        :param module_name: if name passed it will only place that module into the q
        :return: NONE
        """
        if self.config['args'].module:
            # populate only one module in the q
            for mod in modules:
                # populate the q with module data if name hits
                if self.config['args'].module in mod:
                    self.task_queue.put(mod)
        else:
            for mod in modules:
                # populate the q with module data
                self.logger.infomsg('adding module to task queue: ' +
                                    str(mod), 'CoreProcess')
                self.task_queue.put(mod)
        self.module_count = len(modules)
        self._configure_processes(len(modules))

    def clear_task_queue(self):
        """
        If tasked to shutdown, clear the queue for cleanup
        :return: NONE
        """
        # empty the pipe q
        self.logger.infomsg('tasked to clear_task_queue()', 'CoreProcess')
        while not self.task_queue.empty():
            obj = self.task_queue.get()
            del obj
        # allow GC to pick up and flush pipe
        self.logger.infomsg(
            'clear_task_queue() completed empty', 'CoreProcess')
        self.task_queue.close()

    def _pbar_thread(self):
        """
        Built be called from a thread and do simple
        math to watch progress of Dynamic modules.
        :return: NONE
        """
        if not self.config['silent']:
            start_count = len(self.procs)
            self.start_progress_bar(self.module_count)
            while self.check_active():
                try:
                    dm = self.progress_bar_pickup.get()
                except Exception as e:
                    print(e)
                if dm == None:
                    self.close_progress_bar()
                    break
                if dm:
                    if dm[0] == 'complete':
                        self.progress_print(self.blue_text(dm[1]))
                        self.inc_progress_bar(1)
                    if dm[0] == 'execute':
                        self.progress_print(self.blue_text(dm[1]))
                if self.progress_bar_pickup.empty():
                    time.sleep(0.1)

    def _start_thread_function(self, pointer):
        """
        starts a late or early thread.
        :param pointer: Function def
        :return: NONE
        """
        # TODO: FIX this hack and use real pointer?
        self.threads.insert(0, threading.Thread(
            target=self._pbar_thread()))

        t = self.threads[0]
        t.start()

    def _start_threads(self):
        """
        Function to handle threads to be spawned.
        :return: NONE
        """
        self.threads.append(threading.Thread(
            target=self._task_output_queue_consumer))

        for t in self.threads:
            # TODO: Fix issue where threads die before job is parsed
            # TODO: Make some threads daemon?
            # t.daemon = True
            t.start()

    def stop_threads(self):
        """
        Attempt to clean up threads before bail.
        :return: NONE
        """
        self.logger.infomsg(
            'tasked to stop_threads() putting (NONE)', 'CoreProcess')
        self.task_output_queue.put(None)
        for t in self.threads:
            self.logger.infomsg(
                'Attempting to shutting down thread in stop_threads()', 'CoreProcess')
            t.join
        self.print_red("[!] All consumer threads have been joined")
        self.logger.infomsg(
            'All consumer threads joined in stop_threads()', 'CoreProcess')

    def join_threads(self):
        """
        Attempt to clean up threads before bail.
        :return: NONE
        """
        self.logger.infomsg(
            'tasked to join_threads() putting (NONE)', 'CoreProcess')
        self.task_output_queue.put(None)
        while True:
            if self.task_output_queue.empty() == True:
                self.logger.infomsg(
                    'self.task_output_queue is empty! breaking loop', 'CoreProcess')
                break
            else:
                self.logger.infomsg(
                    'self.task_output_queue not empty sleeping for 1 second', 'CoreProcess')
                time.sleep(1)
        for t in self.threads:
            t.join

    def start_processes(self):
        """
        Executes all procs with a given module and
        passes it the proper objects to communicate with
        the core run time.

        :param module_obj: Module Ooject
        :param queues: A list of queue objects
        :return: BOOL 
        """
        self._start_threads()
        for _ in range(self.processors):
            self.logger.infomsg(
                'start_processes() is kicking off empty procs with queue objects', 'CoreProcess')
            self.start_process(self.config, self.task_queue,
                               self.task_output_queue, self.progress_bar_pickup)
        for p in self.procs:
            p.daemon = True
            p.start()
            self.logger.infomsg(
                'start_process() started proc with daemon mode', 'CoreProcess')

    def start_process(self, config, task_queue, task_output_queue, progress_bar_pickup):
        """
        Executes a proc with a given module and
        passes it the proper objects to communicate with
        the core run time.

        :param config: Module Ooject
        :param task_queue: A list of queue objects
        :param task_output_queue:
        :return: BOOL 
        """
        # add all process to a list so we itt over them
        queue_dict = {
            'task_queue': task_queue,
            'task_output_queue': task_output_queue,
            'progress_bar_pickup': progress_bar_pickup
        }
        self.logger.infomsg('start_process() built queue_dict and appending to self.mp.procs, current proc count: '
                            + str(len(self.procs)), 'CoreProcess')
        self.procs.append(
            self.mp.Process(target=self.execute_processes,
                            args=(config, queue_dict, self.modules)))

    def execute_processes(self, config, queue_dict, modules):
        """
        Executes the module required and passed.
        :param module_obj: module settings
        :param queues: passed list obj
        :return: 
        """
        while True:
            # loop to execute taskings from taskQ
            q = queue_dict['task_queue'].get()
            pbq = queue_dict['progress_bar_pickup']
            if q == None:
                self.logger.infomsg(
                    'execute_processes() dynamic module task_queue empty, EXITING process', 'CoreProcess')
                break
            dynamic_module = modules[q]
            try:
                dm = dynamic_module.DynamicModule(config)
                self.logger.infomsg('execute_processes() starting module: '
                                    + str(dm.info['Name']), 'CoreProcess')
                msg = "Executing module: %s %s" % ('{0: <22}'.format(
                    "("+dm.info['Module']+")"), "("+dm.info['Name']+")")
                if not self.config['silent']:
                    pbq.put(['execute', msg])
                # blocking
                dm.dynamic_main(queue_dict)
                self.logger.infomsg('execute_processes() completed module: '
                                    + str(dm.info['Name']), 'CoreProcess')
                msg = "Module completed: %s %s" % (
                    '{0: <22}'.format("(" + dm.info['Module'] + ")"), "(" + dm.info['Name'] + ")")
                if not self.config['silent']:
                    pbq.put(['complete', msg])
            except Exception as e:
                self.logger.warningmsg('execute_processes hit fatal error: '
                                       + str(e), 'CoreProcess')
                self.print_red(" [!] Module process failed: %s %s" % (
                    '{0: <22}'.format("(" + dm.info['Module'] + ")"), "(" + e + ")"))

    def execute_process(self, mod_name, config, queue_dict):
        """
        Executes the module required and passed.
        :param module_obj: module settings
        :param queues: passed list obj
        :return:
        """
        static_module = self.static_modules[mod_name]
        try:
            sm = static_module.DynamicModule(config)
            if not self.config['silent']:
                self.print_green(" [*] Executing module: %s %s" % (
                    '{0: <22}'.format("("+sm.info['Module']+")"), "("+sm.info['Name']+")"))
            sm.dynamic_main(queue_dict)
            if not self.config['silent']:
                self.print_green(" [*] Module completed: %s %s" % (
                    '{0: <22}'.format("(" + sm.info['Module'] + ")"), "(" + sm.info['Name'] + ")"))
        except Exception as e:
            if not self.config['silent']:
                self.print_red(" [!] Module process failed: %s %s" % (
                    '{0: <22}'.format("(" + sm.info['Module'] + ")"), "(" + str(e) + ")"))

    def check_active_len(self):
        """
        Checks for active pids and returns count.
        :return: int
        """
        _value = len(self.mp.active_children())
        self.logger.infomsg('check_active_len() checking current active pids: '
                            + str(_value), 'CoreProcess')
        return _value

    def check_active(self):
        """
        Check if mp is has active pids
        :return: BOOL
        """
        if len(self.mp.active_children()):
            self.logger.infomsg(
                'check_active() for MP pids: True', 'CoreProcess')
            return True
        else:
            self.logger.infomsg(
                'check_active() for MP pids: False', 'CoreProcess')
            return False

    def join_processes(self):
        """
        Attempt to join all the process to clean 
        them up.
        :return: NONE
        """
        for p in self.procs:
            self.logger.infomsg('join_processes() starting to join self.procs, current active: '
                                + str(len(self.procs)), 'CoreProcess')
            p.join()

    def list_processes(self):
        """
        List all procs and pids.
        :return: NONE
        """
        for p in self.procs:
            pid = p.pid
            p_name = p.name
            self.print_yellow(
                "[!] Process info: (PID: %s) (NAME: %s)" % (str(pid), str(p_name)))

    def list_processes_exitcode(self):
        """
        List all procs and exitcode
        :return: 
        """
        for p in self.procs:
            pid = p.pid
            ec = p.exitcode
            self.print_yellow(
                "[!] Process info: (PID: %s) (EXITCODE: %s)" % (str(pid), str(ec)))

    def kill_processes(self):
        """
        Attempt to kill all child pids
        and clean up from CTRL+C.
        :return: NONE
        """
        for p in self.procs:
            pid = p.pid
            p_name = p.name
            while p.is_alive():
                p.terminate()
            self.print_red("[!] Process has been terminated: (PID: %s) (NAME: %s)" % (
                str(pid), str(p_name)))



from unittest import TestCase
from src import core_printer


class TestCorePrinters(TestCase):
    p = core_printer.CorePrinters()

    def test_blue_text(self):
        msg1 = self.p.blue_text("test")
        msg2 = "\x1b[34m [*] \x1b[0mtest"
        self.assertEqual(msg1, msg2)

    def test_green_text(self):
        msg1 = self.p.green_text("test")
        msg2 = "\x1b[32m [+] \x1b[0mtest"
        self.assertEqual(msg1, msg2)

    def test_print_entry(self):
        self.p.print_entry()

    def test_print_d_module_start(self):
        self.p.print_d_module_start()

    def test_print_s_module_start(self):
        self.p.print_s_module_start()

    def test_print_config_start(self):
        self.p.print_config_start()

    def test_print_modules(self):
        self.p.print_modules(['modules/bing_search.py'])



import multiprocessing as mp


class ModuleMultiProcessing(object):

    """
    A multiprocessing class to handle execution for all modules
    requiring multiprocessing. Use module* before all items to we dont
    crush other mp classes.
    """

    def __init__(self):
        """
        Create objects used in class.
        """
        self.module_procs = []
        self.module_threads = []
        self.module_processors = mp.cpu_count()
        self.module_mp = mp

    def _module_configure_mp(self):
        """
        Sets the configuration props for
        mp to handle taskings.
        :return: NONE
        """
        # use SPAWN since FORK is not supported on windows
        # SPAWN is slower since it creates a entire python
        # interpter
        self.module_mp.set_start_method('spawn')

    def module_join_processes(self):
        """
        Attempt to join all the process to clean
        them up.
        :return: NONE
        """
        for p in self.module_procs:
            p.join()

    def modue_list_processes(self):
        """
        List all procs and pids.
        :return: NONE
        """
        for p in self.module_procs:
            pid = p.pid
            p_name = p.name
            print("[!] Process info: (PID: %s) (NAME: %s)" %
                  (str(pid), str(p_name)))

    def module_check_active(self):
        """
        Check if mp is has active pids
        :return: BOOL
        """
        if len(self.module_mp.active_children()):
            return True
        else:
            return

    def module_start_processes(self):
        """
        Executes all procs with a given module and
        passes it the proper objects to communicate with
        the core run time.

        :param module_obj: Module Ooject
        :param queues: A list of queue objects
        :return: BOOL
        """
        for _ in range(self.module_processors):
            self.module_start_process()
        for p in self.module_procs:
            p.daemon = True
            p.start()

    def module_start_process(self, function_pointer, queue_list):
        """
        Executes a proc with a given module and
        passes it the proper objects to communicate with
        the core run time.

        :param function_pointer: Module Object
        :return: BOOL
        """
        self.module_procs.append(
            self.module_mp.Process(target=function_pointer, args=(queue_list,)))



class DynamicModule(object):
    """
    Dynamic module class that will be loaded and called 
    at runtime. This will allow modules to easily be independent of the 
    core runtime.
    """

    def __init__(self, json_entry):
        """
        Init class structure. Each module takes a JSON entry object which 
        can pass different values to the module with out changing up the API.
        adapted form  Empire Project:
        https://github.com/EmpireProject/Empire/blob/master/lib/modules/python_template.py

        :param json_entry: JSON data object passed to the module.
        """
        self.json_entry = json_entry
        self.info = {
            # name of the module to be used
            'Name': 'Template enumerator module',

            # version of the module to be used
            'Version': '0.0',

            # description
            'Description': ('Template module to model after',
                            'while using 2 lines'),

            # authors or sources to be quoted
            'Authors': ['@Killswitch-GUI', '@Killswitch-GUI'],

            # list of resources or comments
            'comments': [
                'Please make sure all of the required fields are filled in.',
                'Http://www.google.com'
            ]
        }

        self.options = {
            # options for the module to use by default these will be dynamic in nature

            # threads for the module to use
            'Threads': 10

        }

    def dynamic_main(self, queue_dict):
        """
        Main entry point for process to call.
        :return: 
        """



# Loads modules into the system
import os
import glob
import warnings
import importlib


class LoadModules(object):
    """
    Loads modules from the "modules" directory allowing
    operators to drop modules into the folder.
    """

    def __init__(self):
        """
        Create objects used in class.
        """
        self.modules = {}
        self.dmodules = {}
        self.static_modules = {}
        self.static_dmodules = {}
        self.load_dynamic_modules()
        self.load_static_modules()

    def load_dynamic_modules(self):
        """
        Loads modules into static class variables, 
        these can than be referenced easily.
        :return: NONE
        """
        # loop and assign key and name
        current_script_patch = os.path.dirname(os.path.abspath(__file__))
        warnings.filterwarnings('ignore', '.*Parent module*', )
        x = 1
        path = os.path.join('dynamic_modules', '*.py')
        path = current_script_patch + "/" + path
        for name in glob.glob(path):
            if name.endswith(".py") and ("__init__" not in name) \
                    and ("module_template" not in name):
                quick_path = os.path.join(
                    'simplydomain', 'src', 'dynamic_modules', name.split('/')[-1])
                module_name = quick_path.replace("/", ".").rstrip('.py')
                loaded_modules = self.dynamic_import(module_name)
                self.modules[quick_path] = loaded_modules
                self.dmodules[x] = loaded_modules
                x += 1

    def load_static_modules(self):
        """
        Loads modules into static class variables,
        these can than be referenced easily.
        :return: NONE
        """
        # loop and assign key and name
        current_script_patch = os.path.dirname(os.path.abspath(__file__))
        warnings.filterwarnings('ignore', '.*Parent module*', )
        x = 1
        path = os.path.join('static_modules', '*.py')
        path = current_script_patch + "/" + path
        for name in glob.glob(path):
            if name.endswith(".py") and ("__init__" not in name) \
                    and ("module_template" not in name):
                quick_path = os.path.join(
                    'simplydomain', 'src', 'static_modules', name.split('/')[-1])
                module_name = quick_path.replace("/", ".").rstrip('.py')
                loaded_modules = self.dynamic_import(module_name)
                self.static_modules[quick_path] = loaded_modules
                self.static_dmodules[x] = loaded_modules
                x += 1

    def dynamic_import(self, module):
        """
        Loads a module at runtime.
        :param module: module str (name)
        :return: module obj
        """
        return importlib.import_module(module)



from tqdm import tqdm


class CoreProgress(object):

    """
    Core class for dynamic modules and progress bars.
    Also allows to use core printers with tqdm.write()
    """

    def __init__(self):
        """
        INIT class object and define
        statics.
        """
        # self.progress_bar
        pass

    def update_progress_bar(self):
        """
        Adds a completion record to the progress bar.
        :return:
        """
        self.progress_bar.update()

    def close_progress_bar(self):
        """
        Progress bar cleanup
        :return: NONE
        """
        self.progress_bar.close()

    def progress_print(self, msg_obj):
        """
        Prints a message obj to te tqdm loop.
        :param msg_obj: A obj to be be printed
        :return: NONE
        """
        self.progress_bar.write(msg_obj)

    def start_progress_bar(self, count, maxinterval=1, mininterval=0):
        """
        Start up pbar and show on screen.
        :param count: count of modules
        :return: NONE
        """
        self.progress_bar = tqdm(
            total=count, unit="module", maxinterval=maxinterval, mininterval=mininterval)

    def inc_progress_bar(self, size=0):
        """
        Incs the progrss by (1)
        :param size: If size pass size, otherwise do min inc.
        :return: NONE
        """
        if size:
            self.progress_bar.update(size)
        else:
            self.progress_bar.update()



from unittest import TestCase
from src import module_resolvers


class TestDnsServers(TestCase):

    m = module_resolvers.DnsServers()
    m.populate_servers()

    def test_populate_servers(self):
        a = self.m.nameserver_ips
        if '8.8.8.8' not in a:
            self.fail("missing google resolver in list!")

    def test_populate_config(self):
        j = {}
        j = self.m.populate_config(j)
        if '8.8.8.8' not in j['resolvers']:
            self.fail('resolver json missing google')

    def test_clean_servers(self):
        self.m.clean_servers()
        if not self.m.nameserver_ips:
            self.fail()

    def test_count_resolvers(self):
        if self.m.count_resolvers() < 1:
            self.fail('no resolvers populated')



from setuptools import setup, find_packages


VERSIONFILE = open("VERSION").read()

setup(name='simplydomain',
      version=VERSIONFILE,
      description='simplydomain is a very basic framework to automate domain brute forcing.',
      url='http://github.com/SimplySecurity/simplydomain-pkg',
      author='Alexander Rymdeko-Harvey',
      author_email='a.rymdekoharvey@obscuritylabs.com',
      license='BSD 3.0',
      packages=[
          'simplydomain',
          'simplydomain.src',
          'simplydomain.src.dynamic_modules',
          'simplydomain.src.static_modules',
          'simplydomain.tests'
      ],
      classifiers=[
          # How mature is this project? Common values are
          #   3 - Alpha
          #   4 - Beta
          #   5 - Production/Stable
          'Development Status :: 4 - Beta',
          # Specify the Python versions you support here. In particular, ensure
          # that you indicate whether you support Python 2, Python 3 or both.
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7'
      ],
      install_requires=[
          'aiodns',
          'beautifulsoup4',
          'crtsh',
          'dnsdumpster',
          'fake_useragent',
          'json2xml',
          'requests',
          'setuptools',
          'termcolor',
          'tqdm',
          # Hold uv until windows support: 'uvloop',
          'validators',
          'click'
      ],
      scripts=[
          'simplydomain/bin/simply_domain.py'
      ],
      include_package_data=True,
      zip_safe=False)



from simplydomain.src import core_printer
from simplydomain.src import core_runtime
from simplydomain.src import module_resolvers
from simplydomain.src import core_logger

# import json config..
from simplydomain import config

import os
import sys
import json
import logging
import argparse

##### SIMPLYDOMAIN API FUNCTIONS #####

# STATICS
__core_printer = core_printer.CorePrinters()
__core_logger = core_logger.CoreLogging()
__core_dns_servers = module_resolvers.DnsServers()

"""
Current JSON config struc:
wordlist_bruteforce: BOOL
wordlist_count: INT
raw_bruteforce: BOOL
raw_depth: INT
verbose: BOOL
debug: BOOL
"""


def __raw_depth_check(value):
    """Check if value is positive int.
    
    Arguments:
        value {number} -- value to check
    
    Returns:
        number -- value that was checked
    
    Raises:
        argparse.ArgumentTypeError -- is an invalid positive int value
        argparse.ArgumentTypeError -- is too large of a keyspace for raw depth
    """
    ivalue = int(value)
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(
            "%s is an invalid positive int value" % value)
    if ivalue >= 6:
        raise argparse.ArgumentTypeError(
            "%s is too large of a keyspace for raw depth" % value)
    return ivalue


def __load_dns(config):
    __core_dns_servers.populate_servers()
    __core_dns_servers.populate_config(config)


def __load_config():
    return config.__json_config


def __set_logging(value='INFO'):
    if value == 'CRITICAL':
        __core_logger.start(logging.CRITICAL)
    if value == 'ERROR':
        __core_logger.start(logging.ERROR)
    if value == 'WARNING':
        __core_logger.start(logging.WARNING)
    if value == 'INFO':
        __core_logger.start(logging.INFO)
    if value == 'DEBUG':
        __core_logger.start(logging.DEBUG)


def __parse_values(domain, debug, verbose, wordlist_bruteforce, wordlist_count, raw_bruteforce, raw_depth):
    parser = argparse.ArgumentParser()
    parser.add_argument("DOMAIN", help="domain to query")
    # opts
    parser.add_argument("-wb", "--wordlist-bruteforce", help="enable word list bruteforce module",
                        action="store_true")
    parser.add_argument("-wc", "--wordlist-count", help="set the count of the top words to use DEFAULT: 100",
                        action="store", default=100, type=int)
    parser.add_argument("-rb", "--raw-bruteforce", help="enable raw bruteforce module",
                        action="store_true")
    parser.add_argument("-rd", "--raw-depth", help="set the count of depth to raw bruteforce DEFAULT: 3",
                        action="store", default=3, type=__raw_depth_check)
    parser.add_argument("-m", "--module", help="module to hit",
                        action="store")
    parser.add_argument(
        "-o", "--output", help="output directory location (Ex. /users/test/)")
    parser.add_argument("-on", "--output-name",
                        help="output directory name (Ex. test-2017)",)
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    parser.add_argument("-d", "--debug", help="enable debug logging to .SimplyDns.log file, default WARNING only",
                        action="store_true")
    args = []
    if debug:
        args.append('-d')
    if verbose:
        args.append('-v')
    if wordlist_bruteforce:
        args.append('-wb')
    if wordlist_count:
        args.append('-wc')
        args.append(str(wordlist_count))
    if raw_bruteforce:
        args.append('-rb')
    if raw_depth:
        args.append('-rd')
        args.append(str(raw_depth))
    if domain:
        args.append(str(domain))
    return parser.parse_args(args)


def execute_raw_bruteforce(domain, config={}, dnsservers=[], debug='CRITICAL', verbose=False, wordlist_count=0, return_type='json', wordlist_bruteforce=False, raw_bruteforce=True, raw_depth=2):
    """Executes only the raw bruteforce search function of simplydomain.

    Executes the static raw brute-force module of simplydomain. 
    This allows simplydomain to generate all applicable RFC character 
    sets off a subdomain keyspace. This can range from 1 char() to 5 char() 
    which can feasibly be brute forced.

    Arguments:
        domain {str} -- domain to search

    Keyword Arguments:
        config {dict} -- sets the JSON config settings for the opperation (default: {{}})
        dnsservers {list} -- sets a list of top DNS servers to resolve (default: {[]})
        debug {str} -- sets the python logging setting: DEBUG, INFO, WARNING, ERROR, CRITICAL (default: {'CRITICAL'})
        verbose {bool} -- sets the verbosity count (default: {False})
        wordlist_count {number} -- count to bruteforce from the word list 0-1,000,000 (default: {0})
        return_type {str} -- sets the return output type to (default: {'json'})
        wordlist_bruteforce {bool} -- enable wordlist bruteforce (default: {False})
        raw_bruteforce {bool} -- enable raw bruteforce (default: {True})
        raw_depth {number} -- depth of keyspace (default: {2})

    Returns:
        str -- (dict | json) object
    """
    __set_logging(debug)
    if not config:
        config = __load_config()
    if not dnsservers:
        dnsservers = __load_dns(config)
    config['args'] = __parse_values(
        domain,
        debug,
        verbose,
        wordlist_bruteforce,
        wordlist_count,
        raw_bruteforce,
        raw_depth
    )
    cr = core_runtime.CoreRuntime(__core_logger, config)
    return cr.execute_raw_bruteforce(return_type=return_type)


def execute_wordlist_bruteforce(domain, config={}, dnsservers=[], debug='CRITICAL', verbose=False, wordlist_count=100, return_type='json', wordlist_bruteforce=True, raw_bruteforce=False, raw_depth=0):
    """Executes only the wordlist bruteforce search function of simplydomain.

    Executes the static wordlist brute-force module of simplydomain. 
    This allows simplydomain to get a range() of X subdomains for to be brute-forced. 
    This can range from 1-1 Million words.

    Arguments:
        domain {str} -- domain to search

    Keyword Arguments:
        config {dict} -- sets the JSON config settings for the opperation (default: {{}})
        dnsservers {list} -- sets a list of top DNS servers to resolve (default: {[]})
        debug {str} -- sets the python logging setting: DEBUG, INFO, WARNING, ERROR, CRITICAL (default: {'CRITICAL'})
        verbose {bool} -- sets the verbosity count (default: {False})
        wordlist_count {number} -- count to bruteforce from the word list 0-1,000,000 (default: {100})
        return_type {str} -- sets the return output type to (default: {'json'})
        wordlist_bruteforce {bool} -- enable wordlist bruteforce (default: {True})
        raw_bruteforce {bool} -- enable raw bruteforce (default: {False})
        raw_depth {number} -- depth of keyspace (default: {0})

    Returns:
        str -- (dict | json) object
    """
    __set_logging(debug)
    if not config:
        config = __load_config()
    if not dnsservers:
        dnsservers = __load_dns(config)
    config['args'] = __parse_values(
        domain,
        debug,
        verbose,
        wordlist_bruteforce,
        wordlist_count,
        raw_bruteforce,
        raw_depth
    )
    cr = core_runtime.CoreRuntime(__core_logger, config)
    return cr.execute_bruteforce(return_type=return_type)


def execute_search(domain, config={}, dnsservers=[], debug='CRITICAL', verbose=False, wordlist_bruteforce=True, wordlist_count=100, raw_bruteforce=True, raw_depth=3, return_type='json'):
    """Executes the main search function(s) of simplydomain.

    simplydomain consists of many Dynamic modules and Static modules too
    allow a programmer to search a large subset of sources for subdomains 
    easily. Within the simplydomain API, this concept is broken down into 
    executing a large scale search function, and specific Static modules.

    Arguments:
        domain {str} -- domain to search

    Keyword Arguments:
        config {dict} -- sets the JSON config settings for the opperation (default: {{}})
        dnsservers {list} -- sets a list of top DNS servers to resolve (default: {[]})
        debug {str} -- sets the python logging setting: DEBUG, INFO, WARNING, ERROR, CRITICAL (default: {'CRITICAL'})
        verbose {bool} -- sets the verbosity count (default: {False})
        wordlist_bruteforce {bool} -- enable wordlist bruteforce (default: {True})
        wordlist_count {number} -- count to bruteforce from the word list 0-1,000,000 (default: {100})
        raw_bruteforce {bool} -- enable raw bruteforce  (default: {True})
        raw_depth {number} -- enable raw bruteforce (default: {3})
        return_type {str} -- sets the return output type to  (default: {'json'})

    Returns:
        str -- (dict | json) object
    """
    __set_logging(debug)
    if not config:
        config = __load_config()
    if not dnsservers:
        dnsservers = __load_dns(config)
    # now setup the config file
    # in JSON format for the core
    # to use within simplydomain
    config['args'] = __parse_values(
        domain,
        debug,
        verbose,
        wordlist_bruteforce,
        wordlist_count,
        raw_bruteforce,
        raw_depth
    )
    cr = core_runtime.CoreRuntime(__core_logger, config)
    return cr.execute_amp(return_type=return_type)
